/**
 * The MIT License (MIT)
 *
 * Copyright (c) 2020 foxsofter
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package com.hellobike.flutter.thrio.module

import com.hellobike.flutter.thrio.JsonSerializer
import com.hellobike.flutter.thrio.registry.RegistryMap

internal object ModuleJsonSerializers {

    private const val TAG = "ModuleJsonSerializers"

    val serializers by lazy { RegistryMap<String, JsonSerializer<*>>() }

    fun <T> serializeParams(params: T?): Any? {
        if (params == null) {
            return null
        }
        // 已经序列化过了
        if (params is Map<*, *> && params.containsKey("__thrio_TParams__")) {
            @Suppress("UNCHECKED_CAST")
            return params as Map<String, Any>
        }
        val type = (params as Any).javaClass.toString()
        val value = serializers[type] ?: return params

        @Suppress("UNCHECKED_CAST")
        val serializer = value as JsonSerializer<T>
        val serializeParams: Map<String, Any?>? = serializer(params) ?: return params
        return mutableMapOf<String, Any?>(
            "__thrio_TParams__" to type
        ).apply { if (serializeParams != null) putAll(serializeParams) }
    }
}